#version 150

in vec3 vertexPosition;
in vec3 particlePosition;
in vec4 particleOrientation;
in vec4 particleColor;
in float particleSize;
in vec3 vertexNormal;

uniform vec3 lightPos0, lightPos1;
uniform mat4 projectionMatrix;
uniform vec3 center;

out vec3 lightDir0, normal;
out vec4 modelViewPos, color;

mat4 rotationMatrix(vec3 axis, float angle)
{
	// Source: http://www.neilmendoza.com/glsl-rotation-about-an-arbitrary-axis/
	// Source's source: http://en.wikipedia.org/wiki/Rotation_matrix#Rotation_matrix_from_axis_and_angle
    axis = normalize(axis);
    float s = sin(angle);
    float c = cos(angle);
    float oc = 1.0 - c;
    return mat4(oc * axis.x * axis.x + c,           oc * axis.x * axis.y - axis.z * s,  oc * axis.z * axis.x + axis.y * s,  0.0,
                oc * axis.x * axis.y + axis.z * s,  oc * axis.y * axis.y + c,           oc * axis.y * axis.z - axis.x * s,  0.0,
                oc * axis.z * axis.x - axis.y * s,  oc * axis.y * axis.z + axis.x * s,  oc * axis.z * axis.z + c,           0.0,
                0.0,                                0.0,                                0.0,                                1.0);
}

void main(void)
{
	// Store color
	color = particleColor;
	
	// Rotation
	mat4 rotation = rotationMatrix(particleOrientation.xyz, particleOrientation.w);
	
	// Store normal
	normal = normalize(rotation * vec4(vertexNormal, 1.0)).xyz;
	
	// Store world position
	modelViewPos = vec4(particlePosition, 0.0) + rotation * vec4((vertexPosition - center) * particleSize, 1.0);
	
	// Store light direction
	lightDir0 = normalize((lightPos0 - modelViewPos.xyz));
	
	// Store screen position
	gl_Position = projectionMatrix * modelViewPos;
}